<?php
/**
*  This file is part of the 'Videos' module feature developped for Prestashop  platform.
* 
*  You are not allowed to use it on several site
*  You are not allowed to sell or redistribute this module
*  This header must not be removed
* 
*  @author    Botigues.cat <info@botigues.cat>
*  @copyright 2014 Botigues.cat
*  @version  1.5.1
*/

if (!defined('_PS_VERSION_')) exit;

require_once(_PS_MODULE_DIR_.'videos/videos.php');
    

class AdminVideoCategories extends AdminController
{
	public $bootstrap = true ;
	private $module = NULL;
        
        public static $l_cache;

	public function __construct() {
		$this->module =  new videos();
                $this->image_dir = _PS_IMG_DIR_ . 'videos/';
		$this->context = Context::getContext();
                
                $this->table = 'video_category';
                $this->className = 'VideoCategory';
		$this->lang = true;
		$this->addRowAction('edit');
		$this->addRowAction('delete');
		$this->bulk_actions = array('delete' => array('text' => $this->l('Delete selected'), 'confirm' => $this->l('Delete selected items?')));
                
		$this->fields_list = array(
                    'id_video_category' => array('title' => $this->l('ID'), 'align' => 'center'),
                    'name' => array('title' => $this->l('Name')),
                    'description' => array('title' => $this->l('Description'),	'callback' => 'getDescriptionClean',),
                    'active' => array('title' => $this->l('Enabled'), 'align' => 'center', 'active' => 'status', 'type' => 'bool')
		);
                
		parent::__construct();
	}

	public function initPageHeaderToolbar()
	{
		if (empty($this->display))
		{
			$this->page_header_toolbar_btn['new_video_category'] = array(
				'href' => self::$currentIndex.'&addvideo_category&token='.$this->token,
				'desc' => $this->l('Add new video category', null, null, false),
				'icon' => 'process-icon-new'
			);
		}

		parent::initPageHeaderToolbar();
	}

	public function renderForm()
	{
		if (!$this->loadObject(true))
			return;
		
		if (Validate::isLoadedObject($this->object))
			$this->display = 'edit';
		else
			$this->display = 'add';

		$this->toolbar_btn['save-and-stay'] = array(
			'short' => 'SaveAndStay',
			'href' => '#',
			'desc' => $this->l('Save and stay'),
		);
		$this->initToolbar();
                
		$this->fields_form = array(
			'legend' => array(
				'title' => $this->l('Video'),
				'image' => '../img/admin/tab-categories.gif'
			),
			'input' => array(
				array(
					'type' => 'text',
					'label' => $this->l('Name'),
					'name' => 'name',
					'lang' => true,
					'required' => true,
					'class' => 'copy2friendlyUrl',
					'hint' => $this->l('Invalid characters:').' <>;=#{}',
					'size' => 50
				),
				array(
					'type' => 'textarea',
					'label' => $this->l('Description'),
					'name' => 'description',
					'lang' => true,
					'size' => 50,
					'rows' => 10,
					'cols' => 100,
                                        'autoload_rte' => true,
				),
				array(
					'type' => 'text',
					'label' => $this->l('Meta title:'),
					'name' => 'meta_title',
					'lang' => true,
                                        'desc' => $this->l('Public title for the product page, and for search engines. Leave blank to use the category name.')
				),
				array(
					'type' => 'text',
					'label' => $this->l('Meta description:'),
					'name' => 'meta_description',
					'lang' => true,
                                        'desc' => $this->l('This description will appear in search engines. You need a single sentence, shorter than 160 characters (including spaces).')
				),
				array(
					'type' => 'text',
					'label' => $this->l('Friendly URL:'),
					'name' => 'link_rewrite',
					'lang' => true,
					'required' => true,
					'hint' => $this->l('Only letters and the minus (-) character are allowed'),
                                        'desc' => $this->l('This is the human-readable URL, as generated from the category name. You can change it if you want.')
				),
				array(
					'type' => 'radio',
					'label' => $this->l('Displayed:'),
					'name' => 'active',
					'required' => false,
					'class' => 't',
					'is_bool' => true,
					'values' => array(
						array(
							'id' => 'active_on',
							'value' => 1,
							'label' => $this->l('Enabled')
						),
						array(
							'id' => 'active_off',
							'value' => 0,
							'label' => $this->l('Disabled')
						)
					),
				),
			),
			'submit' => array(
				'title' => $this->l('Save'),
				'class' => 'button'
			)
		);
		$this->tpl_form_vars['PS_ALLOW_ACCENTED_CHARS_URL'] = (int)Configuration::get('PS_ALLOW_ACCENTED_CHARS_URL');
                
		return parent::renderForm();
	}

	/**
	  * Allows to display the category description without HTML tags and slashes
	  *
	  * @return string
	  */
	public static function getDescriptionClean($description)
	{
		return strip_tags(stripslashes($description));
	}
        
        
        // translations
        protected function l($string, $class = 'AdminVideoCategories', $addslashes = FALSE, $htmlentities = TRUE)
	{
		// need to be called in order to populate $classInModule
		$str = self::findTranslation($this->module->name, $string, $class);
		$str = $htmlentities ? htmlentities($str, ENT_QUOTES, 'utf-8') : $str;
		return str_replace('"', '&quot;', ($addslashes ? addslashes($str) : stripslashes($str)));
	}

	public static function findTranslation($name, $string, $source)
	{
		static $_MODULES, $_MODULE;
		if (!is_array($_MODULES))
		{
			// note: $_COOKIE[iso_code] is set in createCustomToken();
			$file = _PS_MODULE_DIR_.$name.'/translations/'.Context::getContext()->language->iso_code.'.php';
			if (file_exists($file) && include($file))
				$_MODULES = !empty($_MODULES)?array_merge($_MODULES, $_MODULE):$_MODULE;
		}
		$cache_key = $name.'|'.$string.'|'.$source;

		if (!isset(self::$l_cache[$cache_key]))
		{
			if (!is_array($_MODULES))
				return $string;
			// set array key to lowercase for 1.3 compatibility
			$_MODULES = array_change_key_case($_MODULES);
			if (defined('_THEME_NAME_'))
				$currentKey = '<{'.strtolower($name).'}'.strtolower(_THEME_NAME_).'>'.strtolower($source).'_'.md5($string);
			else
				$currentKey = '<{'.strtolower($name).'}default>'.strtolower($source).'_'.md5($string);
			// note : we should use a variable to define the default theme (instead of "prestashop")
			$defaultKey = '<{'.strtolower($name).'}prestashop>'.strtolower($source).'_'.md5($string);
			$currentKey = $defaultKey;

			if (isset($_MODULES[$currentKey]))
				$ret = stripslashes($_MODULES[$currentKey]);
			elseif (isset($_MODULES[strtolower($currentKey)]))
				$ret = stripslashes($_MODULES[strtolower($currentKey)]);
			elseif (isset($_MODULES[$defaultKey]))
				$ret = stripslashes($_MODULES[$defaultKey]);
			elseif (isset($_MODULES[strtolower($defaultKey)]))
				$ret = stripslashes($_MODULES[strtolower($defaultKey)]);
			else
				$ret = stripslashes($string);

			self::$l_cache[$cache_key] = $ret;
		}
		return self::$l_cache[$cache_key];
	}
     
}

