<?php
/**
*  This file is part of the 'Videos' module feature developped for Prestashop  platform.
* 
*  You are not allowed to use it on several site
*  You are not allowed to sell or redistribute this module
*  This header must not be removed
* 
*  @author    Botigues.cat <info@botigues.cat>
*  @copyright 2014 Botigues.cat
*  @version  1.5.1
*/

if (!defined('_PS_VERSION_')) exit;

require_once(_PS_MODULE_DIR_.'videos/videos.php');


class AdminVideos extends AdminController
{
	public $bootstrap = true;
	protected $categories_array = array();
	protected $maxFileSize  = 2000000;
	private $module = null;

	public static $l_cache;

	public function __construct()
	{
		$this->module = new videos();
		$this->image_dir = _PS_IMG_DIR_.'videos/';
		$this->context = Context::getContext();

		$this->table = 'video';
		$this->className = 'Video';
		$this->lang = true;
		$this->addRowAction('edit');
		$this->addRowAction('delete');
		$this->bulk_actions = array('delete' => array('text' => $this->l('Delete selected'), 'confirm' => $this->l('Delete selected items?')));

		$this->fields_list = array(
			'id_video' => array('title' => $this->l('ID'), 'align' => 'center'),
			'name' => array('title' => $this->l('Name')),
			'description_short' => array('title' => $this->l('Description')),
			'active' => array('title' => $this->l('Enabled'), 'align' => 'center', 'active' => 'status', 'type' => 'bool')
		);

		parent::__construct();
	}

	public function initPageHeaderToolbar()
	{
		if (empty($this->display))
		{
			$this->page_header_toolbar_btn['new_video'] = array(
				'href' => self::$currentIndex.'&addvideo&token='.$this->token,
				'desc' => $this->l('Add new video', null, null, false),
				'icon' => 'process-icon-new'
			);
		}

		parent::initPageHeaderToolbar();
	}

	public function renderForm()
	{
		if (!$this->loadObject(true))
			return;

		if (Validate::isLoadedObject($this->object))
			$this->display = 'edit';
		else
			$this->display = 'add';

		$this->toolbar_btn['save-and-stay'] = array(
			'short' => 'SaveAndStay',
			'href' => '#',
			'desc' => $this->l('Save and stay'),
		);
		$this->initToolbar();

		$this->fields_form = array(
			'legend' => array(
				'title' => $this->l('Video'),
				'image' => '../modules/videos/img/tab.gif'
			),
			'input' => array(
				array(
					'type' => 'text',
					'label' => $this->l('Name'),
					'name' => 'name',
					'lang' => true,
					'required' => true,
					'class' => 'copy2friendlyUrl',
					'hint' => $this->l('Invalid characters:').' <>;=#{}',
					'size' => 50
				),
				array(
					'type' => 'select',
					'label' => $this->l('Video Type'),
					'name' => 'type',
					'required' => true,
					'options' => array(
						'query' => array(
							array('value' => 'youtube', 'name' => $this->l('Youtube')),
							array('value' => 'vimeo', 'name' => $this->l('Vimeo')),
						),
						'id' => 'value',
						'name' => 'name',
					)
				),
				array(
					'type' => 'text',
					'label' => $this->l('Video Code'),
					'name' => 'code',
					'required' => true,
					'size' => 50,
					'desc' => $this->l('Copy the CODE that appears at the Video URL. For example:').'<br />'.
							$this->l('For Youtube: https://www.youtube.com/watch?v=CODE').'<br />'.
							$this->l('For Vimeo: http://vimeo.com/CODE')
				),
				array(
					'type' => 'textarea',
					'label' => $this->l('Description Short'),
					'name' => 'description_short',
					'lang' => true,
					'size' => 50,
					'rows' => 10,
					'cols' => 100,
					'desc' => $this->l('This description appears in video lists as a video resume.')
				),
				array(
					'type' => 'textarea',
					'label' => $this->l('Description'),
					'name' => 'description',
					'lang' => true,
					'size' => 50,
					'rows' => 10,
					'cols' => 100,
					'autoload_rte' => true,
				),
				array(
					'type' => 'file',
					'label' => $this->l('Image:'),
					'name' => 'image',
					'display_image' => true,
					'desc' => $this->l('Upload a image from your computer.')
				),
				array(
					'label' => $this->l('Related Products:'),
					'name' => 'inputRelatedProducts',
					'type' => ''
				),
				array(
					'label' => $this->l('Categories:'),
					'name' => 'categories',
					'type' => ''
				),
				array(
					'type' => 'text',
					'label' => $this->l('Meta title:'),
					'name' => 'meta_title',
					'lang' => true,
					'desc' => $this->l('Public title for the product page, and for search engines. Leave blank to use the video name.')
				),
				array(
					'type' => 'text',
					'label' => $this->l('Meta description:'),
					'name' => 'meta_description',
					'lang' => true,
					'desc' => $this->l('This description will appear in search engines. You need a single sentence, shorter than 160 characters (including spaces).')
				),
				array(
					'type' => 'text',
					'label' => $this->l('Friendly URL:'),
					'name' => 'link_rewrite',
					'lang' => true,
					'required' => true,
					'hint' => $this->l('Only letters and the minus (-) character are allowed'),
					'desc' => $this->l('This is the human-readable URL, as generated from the video name. You can change it if you want.')
				),
				array(
					'type' => 'radio',
					'label' => $this->l('Displayed:'),
					'name' => 'active',
					'required' => false,
					'class' => 't',
					'is_bool' => true,
					'values' => array(
						array(
							'id' => 'active_on',
							'value' => 1,
							'label' => $this->l('Enabled')
						),
						array(
							'id' => 'active_off',
							'value' => 0,
							'label' => $this->l('Disabled')
						)
					),
				),
			),
			'submit' => array(
				'title' => $this->l('Save'),
				'class' => 'button'
			)
		);

		$this->tpl_form_vars['PS_ALLOW_ACCENTED_CHARS_URL'] = (int)Configuration::get('PS_ALLOW_ACCENTED_CHARS_URL');

		if (!($obj = $this->loadObject(true)))
			return;

		if ($obj->id)
		{
			$image = ImageManager::thumbnail(_PS_IMG_DIR_.'videos/'.$obj->id.'.jpg', $this->table.'_'.(int)$obj->id.'.jpg', 350, 'jpg', true, true);

			if (version_compare(_PS_VERSION_, '1.6.0.0') >= 0)
			{
				$this->fields_form['input'][5]['image'] = $image ? $image : false;
				$this->fields_form['input'][5]['size'] = $image ? filesize(_PS_IMG_DIR_.'videos/'.$obj->id.'.jpg') / 1000 : false;
			}
			else
			{
				$this->fields_value['image'] = array(
					'image' => $image ? $image : false,
					'size' => $image ? filesize(_PS_IMG_DIR_.'videos/'.$obj->id.'.jpg') / 1000 : false
				);
			}
		}

		return parent::renderForm();
	}

	public function processSave()
	{
		parent::processSave();

		if (!isset($this->errors) || empty($this->errors))
		{
			if (isset($_FILES) && !empty($_FILES))
			{
				foreach ($_FILES as $key => $img)
				{
					if ($img['tmp_name'] != '')
					{
						switch ($key)
						{
							case 'image':
								$this->object->deleteImage();
								$this->object->saveImage($key);
								break;

							default:
								break;
						}
					}
				}
			}

			$this->updateRelatedProducts($this->object);
			$this->updateCategories($this->object);

		}
	}

	/**
	 * postProcess handle every checks before saving products information
	 *
	 * @return void
	 */
	public function postProcess()
	{
		parent::postProcess();

		if ($this->display == 'edit' || $this->display == 'add')
		{
			$this->addjQueryPlugin(array(
				'autocomplete'
			));
		}
	}

	public function initContent()
	{
		if ($this->display == 'edit' || $this->display == 'add')
		{
			$post_products = array();
			$post_categories = array();

			if ($this->className)
				$this->loadObject(true);
			if (isset($this->object))
			{
				// RelatedProducts block
				$post_products = Video::getRelatedProductsLight($this->context->language->id, $this->object->id);
				// Categories block
				$categories = Video::getCategories($this->context->language->id, $this->object->id);
				foreach ($categories as $post_category)
					$post_categories[] = $post_category['id_video_category'];
			}

			$this->context->smarty->assign(array(
				'products'=> $post_products,
				'post_categories'=> $post_categories,
				'categories'=> VideoCategory::getVideoCategories($this->context->language->id, false)
			));
		}
		parent::initContent();
	}

	public function updateCategories($video)
	{
		$video->deleteCategories();
		if ($categories = Tools::getValue('categories'))
		{
			if (!empty($categories))
				$video->addCategories($categories);
		}
	}

	public function updateRelatedProducts($video)
	{
		$video->deleteRelatedProducts();
		if ($products = Tools::getValue('inputRelatedProducts'))
		{
			$products_id = array_unique(explode('-', $products));
			if (!empty($products_id))
			{
				array_pop($products_id);
				$video->addRelatedProducts($products_id);
			}
		}
	}

	public function haveThisProduct($product_id, $products)
	{
		foreach ($products as $product)
			if ((int)$product['id_product'] == (int)$product_id)
				return true;
		return false;
	}


	/* translations */
	protected function l($string, $class = 'AdminVideos', $addslashes = false, $htmlentities = true)
	{
		// need to be called in order to populate $classInModule
		$str = self::findTranslation($this->module->name, $string, $class);
		$str = $htmlentities ? htmlentities($str, ENT_QUOTES, 'utf-8') : $str;
		return str_replace('"', '&quot;', ($addslashes ? addslashes($str) : Tools::stripslashes($str)));
	}

	public static function findTranslation($name, $string, $source)
	{
		static $_MODULES, $_MODULE;
		if (!is_array($_MODULES))
		{
			// note: $_COOKIE[iso_code] is set in createCustomToken();
			$file = _PS_MODULE_DIR_.$name.'/translations/'.Context::getContext()->language->iso_code.'.php';
			if (file_exists($file) && include($file))
			$_MODULES = !empty($_MODULES)?array_merge($_MODULES, $_MODULE):$_MODULE;
		}
		$cache_key = $name.'|'.$string.'|'.$source;

		if (!isset(self::$l_cache[$cache_key]))
		{
			if (!is_array($_MODULES))
				return $string;
			// set array key to lowercase for 1.3 compatibility
			$_MODULES = array_change_key_case($_MODULES);
			if (defined('_THEME_NAME_'))
				$currentKey = '<{'.Tools::strtolower($name).'}'.Tools::strtolower(_THEME_NAME_).'>'.Tools::strtolower($source).'_'.md5($string);
			else
				$currentKey = '<{'.Tools::strtolower($name).'}default>'.Tools::strtolower($source).'_'.md5($string);
			// note : we should use a variable to define the default theme (instead of "prestashop")
			$defaultKey = '<{'.Tools::strtolower($name).'}prestashop>'.Tools::strtolower($source).'_'.md5($string);
			$currentKey = $defaultKey;

			if (isset($_MODULES[$currentKey]))
				$ret = Tools::stripslashes($_MODULES[$currentKey]);
			elseif (isset($_MODULES[Tools::strtolower($currentKey)]))
				$ret = Tools::stripslashes($_MODULES[Tools::strtolower($currentKey)]);
			elseif (isset($_MODULES[$defaultKey]))
				$ret = Tools::stripslashes($_MODULES[$defaultKey]);
			elseif (isset($_MODULES[Tools::strtolower($defaultKey)]))
				$ret = Tools::stripslashes($_MODULES[Tools::strtolower($defaultKey)]);
			else
				$ret = Tools::stripslashes($string);

			self::$l_cache[$cache_key] = $ret;
		}
		return self::$l_cache[$cache_key];
	}

}
