<?php
/**
*  This file is part of the 'Videos' module feature developped for Prestashop  platform.
* 
*  You are not allowed to use it on several site
*  You are not allowed to sell or redistribute this module
*  This header must not be removed
* 
*  @author    Botigues.cat <info@botigues.cat>
*  @copyright 2014 Botigues.cat
*  @version  1.5.1
*/

class Video extends ObjectModel
{
	
	public $id_video;     
	public $name;
	public $active;
	public $code;
	public $type;
	public $description_short;
	public $description;
	public $meta_description;
	public $meta_title;
	public $link_rewrite;

	/**
	* @see ObjectModel::$definition
	*/
	public static $definition = array(
			'table' => 'video',
			'primary' => 'id_video',
			'multilang' => true,
			'fields' => array(
			'name' => 		array('type' => self::TYPE_STRING, 'validate' => 'isString', 'lang' => true, 'required' => true, 'size' => 128),
			'type' => 		array('type' => self::TYPE_STRING, 'validate' => 'isString', 'values' => array('youtube', 'vimeo'), 'default' => 'youtube'),
			'code' => 		array('type' => self::TYPE_STRING, 'validate' => 'isString', 'size' => 128),
			'description_short' => 	array('type' => self::TYPE_STRING, 'validate' => 'isString', 'lang' => true, 'size' => 255),
			'description' => 	array('type' => self::TYPE_HTML, 'lang' => true, 'validate' => 'isString'),
			'meta_description' => 	array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'size' => 255),
			'meta_title' => 	array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'size' => 128),
			'link_rewrite' => 	array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isLinkRewrite', 'required' => true, 'size' => 128),

			'active' =>             array('type' => self::TYPE_BOOL)
		),
	);


	public function __construct($id_video = null, $id_lang = null, $id_shop = null)
	{
		parent::__construct($id_video, $id_lang, $id_shop);
		$this->image_dir = _PS_IMG_DIR_.'videos/';

		if (isset($this->id) && file_exists($this->image_dir.$this->id.'.jpg'))
			$this->image = _PS_BASE_URL_._PS_IMG_.'videos/'.$this->id.'.jpg';
		$this->link = $this->getlink($id_lang);
	}


	public function add($autodate = true, $null_values = false)
	{
		if (!parent::add($autodate, $null_values))
			return false;

		return $this->assignToShops();
	}


	public function update($null_values = false)
	{
		if (!parent::update($null_values))
			return false;

		return $this->assignToShops();
	}


	private function assignToShops()
	{
		$data = array();
		$shops = Shop::getCompleteListOfShopsID(); //shops to be imported
		foreach ($shops as $id_shop)
			if (!$this->existsInShop($id_shop))
				$data[] = array(
					self::$definition['primary'] => (int)$this->id,
					'id_shop' => (int)$id_shop,
				);
		return Db::getInstance()->insert(self::$definition['table'].'_shop', $data);
	}


	private function existsInShop($id_shop)
	{
		return (bool)Db::getInstance()->getValue('
			SELECT `'.self::$definition['primary'].'`
			FROM `'._DB_PREFIX_.self::$definition['table'].'_shop`
			WHERE `'.self::$definition['primary'].'` = '.(int)$this->id.'
			AND `id_shop` = '.(int)$id_shop);
	}


	public function delete()
	{
		$this->deleteImage();
		return parent::delete();
	}


	public function deleteImage()
	{
		@unlink($this->image_dir.$this->id.'.jpg');
	}

	public function saveImage($key)
	{
		$img = $_FILES[$key]['tmp_name'];
		$new_file = $this->image_dir.$this->id.'.jpg';

		if (($img_info = getimagesize($img)) === false)
			die('Image not found or not an image');

		$width = $img_info[0];
		$height = $img_info[1];

		switch ($img_info[2])
		{
			case IMAGETYPE_GIF:
				$src = imagecreatefromgif($img);
				break;
			case IMAGETYPE_JPEG:
				$src = imagecreatefromjpeg($img);
				break;
			case IMAGETYPE_PNG:
				$src = imagecreatefrompng($img);
				break;
			default:
				die('Unknown filetype');
		}

		$tmp = imagecreatetruecolor($width, $height);
		imagecopyresampled($tmp, $src, 0, 0, 0, 0, $width, $height, $width, $height);
		imagejpeg($tmp, $new_file);
	}


	public static function getInstanceById($id_video)
	{
		$result = Db::getInstance()->getRow('
			SELECT *
			FROM `'._DB_PREFIX_.'video`
			WHERE `id_video` = '.(int)$id_video
		);

		return $result;
	}


	public static function getVideos($id_lang = false)
	{
		return Db::getInstance()->ExecuteS('
			SELECT *
			FROM `'._DB_PREFIX_.'video` t
			LEFT JOIN `'._DB_PREFIX_.'video_lang` tl ON (t.`id_video` = tl.`id_video`)
			WHERE tl.`id_lang` = '.(int)$id_lang.' AND t.`active` = 1 ORDER BY t.`id_video` DESC');
	}
	
	
	
	
	/****** categories ******/
	public function addCategories($categories_id)
	{
		foreach ($categories_id as $id_category)
			Db::getInstance()->insert('video_categories', array(
				'id_video' => (int)$this->id,
				'id_video_category' => (int)$id_category
			));
	}

	public static function getCategories($id_lang, $id_video)
	{
		$sql = 'SELECT vc.*, vcl.* FROM '._DB_PREFIX_.'video_categories vcs 
			LEFT JOIN `'._DB_PREFIX_.'video_category` vc ON (vcs.`id_video_category`= vc.`id_video_category`)
			LEFT JOIN `'._DB_PREFIX_.'video_category_lang` vcl ON (
				vc.`id_video_category` = vcl.`id_video_category`
				AND vcl.`id_lang` = '.(int)$id_lang.'
			)
			WHERE vcs.`id_video` = '.(int)$id_video;

		return Db::getInstance()->executeS($sql);
	}

	public function deleteCategories()
	{
		return Db::getInstance()->Execute('DELETE FROM `'._DB_PREFIX_.'video_categories` WHERE `id_video` = '.(int)$this->id);
	}
	
	
	
	
	/***** related products *****/
	public function deleteRelatedProducts()
	{
		return Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'video_product` WHERE `id_video` = '.(int)$this->id);
	}

	public static function getRelatedProductsLight($id_lang, $id_video, Context $context = null)
	{
		if (!$context)
			$context = Context::getContext();

		$sql = 'SELECT p.`id_product`, p.`reference`, pl.`name`
			FROM `'._DB_PREFIX_.'video_product` vp
			LEFT JOIN `'._DB_PREFIX_.'product` p ON (p.`id_product`= vp.`id_product`)
			'.Shop::addSqlAssociation('product', 'p').'
			LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (
				p.`id_product` = pl.`id_product`
				AND pl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('pl').'
			)
			WHERE vp.`id_video` = '.(int)$id_video;

		return Db::getInstance()->executeS($sql);
	}

	public function getRelatedProducts($id_lang, $active = true, Context $context = null)
	{
		if (!$context)
			$context = Context::getContext();

		$sql = 'SELECT p.*, pl.*, product_shop.*, stock.out_of_stock, IFNULL(stock.quantity, 0) as quantity,
					MAX(image_shop.`id_image`) id_image, il.`legend`, m.`name` as manufacturer_name, cl.`name` AS category_default,
					DATEDIFF(
						p.`date_add`,
						DATE_SUB(
							NOW(),
							INTERVAL '.(Validate::isUnsignedInt(Configuration::get('PS_NB_DAYS_NEW_PRODUCT')) ? Configuration::get('PS_NB_DAYS_NEW_PRODUCT') : 20).' DAY
						)
					) > 0 AS new
				FROM `'._DB_PREFIX_.'video_product` vp
				LEFT JOIN `'._DB_PREFIX_.'product` p ON p.`id_product` = vp.`id_product`
				'.Shop::addSqlAssociation('product', 'p').'
				LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (
					p.`id_product` = pl.`id_product`
					AND pl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('pl').'
				)
				LEFT JOIN `'._DB_PREFIX_.'category_lang` cl ON (
					product_shop.`id_category_default` = cl.`id_category`
					AND cl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('cl').'
				)
				LEFT JOIN `'._DB_PREFIX_.'image` i ON (i.`id_product` = p.`id_product`)'.
				Shop::addSqlAssociation('image', 'i', false, 'image_shop.cover=1').'
				LEFT JOIN `'._DB_PREFIX_.'image_lang` il ON (i.`id_image` = il.`id_image` AND il.`id_lang` = '.(int)$id_lang.')
				LEFT JOIN `'._DB_PREFIX_.'manufacturer` m ON (p.`id_manufacturer`= m.`id_manufacturer`)
				'.Product::sqlStock('p', 0).'
				WHERE vp.`id_video` = '.(int)$this->id.
				($active ? ' AND product_shop.`active` = 1' : '').'
				GROUP BY product_shop.id_product';
		if (!$result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql))
			return false;
		foreach ($result as &$row)
			$row['id_product_attribute'] = Product::getDefaultAttribute((int)$row['id_product']);
		return Product::getProductsProperties($id_lang, $result);
	}

	public static function getRelatedProductById($product_id)
	{
		return Db::getInstance()->getRow('SELECT `id_product`, `name` FROM `'._DB_PREFIX_.'product_lang` WHERE `id_product` = '.(int)$product_id);
	}

	public function addRelatedProducts($products_id)
	{
		foreach ($products_id as $id_product)
			Db::getInstance()->insert('video_product', array(
				'id_video' => (int)$this->id,
				'id_product' => (int)$id_product
			));
	}
	
	
	
	/****** link ******/
	public function getlink($id_lang, $context = null)
	{
		if (is_null($context) || !($context instanceof Context))
			$context = Context::getContext();

		$languages = Language::getLanguages(true, $context->shop->id);
		$shop_url = $context->shop->getBaseURL();

		if (Configuration::get('PS_REWRITING_SETTINGS'))
		{
			$iso = ($id_lang > 0 && count($languages) > 1) ? Language::getIsoById((int)$id_lang).'/' : '';
			return $shop_url.$iso.'video/'.$this->id_video.'-'.$this->link_rewrite;
		}
		else
		{
			$id_lang = ($id_lang > 0 && count($languages) > 1) ? '&id_lang='.$id_lang : '';
			return $shop_url.'index.php?controller=video&id_video='.$this->id_video.$id_lang;
		}
	}
}

?>
