<?php
/**
*  This file is part of the 'Videos' module feature developped for Prestashop  platform.
* 
*  You are not allowed to use it on several site
*  You are not allowed to sell or redistribute this module
*  This header must not be removed
* 
*  @author    Botigues.cat <info@botigues.cat>
*  @copyright 2014 Botigues.cat
*  @version  1.5.1
*/

class VideoCategory extends ObjectModel
{

	public $id_video_category;
	public $name;
	public $active;
	public $description;
	public $meta_description;
	public $meta_title;
	public $link_rewrite;

	/**
	 * @see ObjectModel::$definition
	 */
	public static $definition = array(
		'table' => 'video_category',
		'primary' => 'id_video_category',
		'multilang' => true,
		'fields' => array(
			'name' => 		array('type' => self::TYPE_STRING, 'validate' => 'isString', 'lang' => true, 'required' => true, 'size' => 32),
			'description' => 	array('type' => self::TYPE_HTML, 'lang' => true, 'validate' => 'isString'),
			'meta_description' => 	array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'size' => 255),
			'meta_title' => 	array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'size' => 128),
			'link_rewrite' => 	array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isLinkRewrite', 'required' => true, 'size' => 128),
			'active' =>             array('type' => self::TYPE_BOOL)
		),
	);


	public function __construct($id_video = null, $id_lang = null, $id_shop = null)
	{
		parent::__construct($id_video, $id_lang, $id_shop);

		$this->link = $this->getlink($id_lang);
	}


	public function add($autodate = true, $null_values = false)
	{
		if (!parent::add($autodate, $null_values))
			return false;

		return $this->assignToShops();
	}


	public function update($null_values = false)
	{
		if (!parent::update($null_values))
			return false;

		return $this->assignToShops();
	}


	private function assignToShops()
	{
		$data = array();
		$shops = Shop::getCompleteListOfShopsID(); //shops to be imported
		foreach ($shops as $id_shop)
			if (!$this->existsInShop($id_shop))
				$data[] = array(
						self::$definition['primary'] => (int)$this->id,
						'id_shop' => (int)$id_shop,
					);
		return Db::getInstance()->insert(self::$definition['table'].'_shop', $data);
	}


	private function existsInShop($id_shop)
	{
		return (bool)Db::getInstance()->getValue('
			SELECT `'.self::$definition['primary'].'`
			FROM `'._DB_PREFIX_.self::$definition['table'].'_shop`
			WHERE `'.self::$definition['primary'].'` = '.(int)$this->id.'
			AND `id_shop` = '.(int)$id_shop);
	}


	public static function getInstanceById($id_video)
	{
		$result = Db::getInstance()->getRow('
			SELECT *
			FROM `'._DB_PREFIX_.'video_category`
			WHERE `id_video_category` = '.(int)$id_video
			);

		return $result;
	}


	public static function getVideoCategories($id_lang = false, $active = true)
	{
		return Db::getInstance()->ExecuteS('
		SELECT *
		FROM `'._DB_PREFIX_.'video_category` vc
		LEFT JOIN `'._DB_PREFIX_.'video_category_lang` vcl ON (vc.`id_video_category` = vcl.`id_video_category`)
		WHERE vcl.`id_lang` = '.(int)$id_lang.
			($active ? ' AND vc.`active` = 1' : '').' ORDER BY vc.`id_video_category` DESC');
	}


	public function getVideos($id_lang = false, $active = true)
	{
		return Db::getInstance()->ExecuteS('
		SELECT v.*, vl.*
		FROM `'._DB_PREFIX_.'video` v
		LEFT JOIN `'._DB_PREFIX_.'video_lang` vl ON (v.`id_video` = vl.`id_video`)
                LEFT JOIN `'._DB_PREFIX_.'video_categories` vcs ON v.`id_video` = vcs.`id_video`
                WHERE vcs.`id_video_category` = '.(int)$this->id.' AND vl.`id_lang` = '.(int)$id_lang.
			($active ? ' AND v.`active` = 1' : '').' ORDER BY v.`id_video` DESC');
	}


	public static function getAllVideos($id_lang = false, $active = true)
	{
		return Db::getInstance()->ExecuteS('
		SELECT v.*, vl.*
		FROM `'._DB_PREFIX_.'video` v
		LEFT JOIN `'._DB_PREFIX_.'video_lang` vl ON (v.`id_video` = vl.`id_video`)
                LEFT JOIN `'._DB_PREFIX_.'video_categories` vcs ON v.`id_video` = vcs.`id_video`
                WHERE vl.`id_lang` = '.(int)$id_lang.
			($active ? ' AND v.`active` = 1' : '').' ORDER BY v.`id_video` DESC');
	}




	/****** link ******/
	public function getlink($id_lang, $p = null, $context = null)
	{
		if (is_null($context) || !($context instanceof Context))
			$context = Context::getContext();

		$languages = Language::getLanguages(true, $context->shop->id);
		$shop_url = $context->shop->getBaseURL();

		if (Configuration::get('PS_REWRITING_SETTINGS'))
		{
			$iso = ($id_lang > 0 && count($languages) > 1) ? Language::getIsoById((int)$id_lang).'/' : '';
			$p = (!is_null($p) ? '?p='.$p : '');

			return $shop_url.$iso.'videos/'.$this->id_video_category.'-'.$this->link_rewrite.$p;
		}
		else
		{
			$id_lang = ($id_lang > 0 && count($languages) > 1) ? '&id_lang='.$id_lang : '';
			$p = (!is_null($p) ? '&p='.$p : '');

			return $shop_url.'index.php?controller=video-category&id_video_category='.$this->id_video_category.$id_lang.$p;
		}
	}

}

?>
