<?php
/**
*  This file is part of the 'Videos' module feature developped for Prestashop  platform.
* 
*  You are not allowed to use it on several site
*  You are not allowed to sell or redistribute this module
*  This header must not be removed
* 
*  @author    Botigues.cat <info@botigues.cat>
*  @copyright 2014 Botigues.cat
*  @version  1.5.1
*/

class VideoControllerCore extends FrontController
{
	public $php_self = 'video';
	protected $video;

	/**
	 * Set default medias for this controller
	 */
	public function setMedia()
	{
		parent::setMedia();

		if (version_compare(_PS_VERSION_, '1.6.0.0') >= 0)
			$this->addCSS(_MODULE_DIR_.'videos/css/video_16.css', 'all');
		else
			$this->addCSS(_MODULE_DIR_.'videos/css/video_15.css', 'all');
	}

	/**
	 * Initialize manufaturer controller
	 * @see FrontController::init()
	 */
	public function init()
	{
		parent::init();

		if ($id_video = Tools::getValue('id_video'))
		{
			$this->video = new Video((int)$id_video, $this->context->language->id);
			if (!Validate::isLoadedObject($this->video) || !$this->video->active || !$this->video->isAssociatedToShop())
			{
				header('HTTP/1.1 404 Not Found');
				header('Status: 404 Not Found');
				$this->errors[] = Tools::displayError('The video does not exist.');
			}
			else
				$this->canonicalRedirection();
		}
		else
			$this->errors[] = Tools::displayError('Missing video ID');
	}

	public function canonicalRedirection($canonical_url = '')
	{
		if (Validate::isLoadedObject($this->video))
			parent::canonicalRedirection($this->video->getLink($this->context->language->id));
	}

	/**
	* Assign template vars related to page content
	* @see FrontController::initContent()
	*/
	public function initContent()
	{
		parent::initContent();

		if (version_compare(_PS_VERSION_, '1.6.0.0') >= 0)
			$this->setTemplate(_PS_ROOT_DIR_.'/modules/videos/views/templates/front/video_16.tpl');
		else
			$this->setTemplate(_PS_ROOT_DIR_.'/modules/videos/views/templates/front/video_15.tpl');

		if (!Validate::isLoadedObject($this->video))
			return;

		$this->context->smarty->assign(array(
			'path' => ($this->video->active ? Tools::safeOutput($this->video->name) : ''),
			'video' => $this->video,
			'meta_title' => (isset($this->video->meta_title) && $this->video->meta_title != '' ? $this->video->meta_title : $this->video->name),
			'meta_description' => (isset($this->video->meta_desription) && $this->video->meta_description != '' ? $this->video->meta_description : $this->video->description_short),
			'products' => $this->video->getRelatedProducts($this->context->language->id)
		));
	}

}
