<?php
/**
*  This file is part of the 'Videos' module feature developped for Prestashop  platform.
* 
*  You are not allowed to use it on several site
*  You are not allowed to sell or redistribute this module
*  This header must not be removed
* 
*  @author    Botigues.cat <info@botigues.cat>
*  @copyright 2014 Botigues.cat
*  @version  1.5.1
*/

class VideoCategoryControllerCore extends FrontController
{
	public $php_self = 'video-category';
	protected $category;

	/**
	 * Set default medias for this controller
	 */
	public function setMedia()
	{
		parent::setMedia();

		if (version_compare(_PS_VERSION_, '1.6.0.0') >= 0)
			$this->addCSS(_MODULE_DIR_.'videos/css/video-category_16.css', 'all');
		else
			$this->addCSS(_MODULE_DIR_.'videos/css/video-category_15.css', 'all');
	}

	/**
	 * Initialize manufaturer controller
	 * @see FrontController::init()
	 */
	public function init()
	{
		parent::init();

		if ($id_video_category = Tools::getValue('id_video_category'))
		{
			$this->category = new VideoCategory((int)$id_video_category, $this->context->language->id);
			if (!Validate::isLoadedObject($this->category) || !$this->category->isAssociatedToShop())
			{
				header('HTTP/1.1 404 Not Found');
				header('Status: 404 Not Found');
				$this->errors[] = Tools::displayError('The video category does not exist.');
			}
			else
				$this->canonicalRedirection();
//		} else {
//			$this->errors[] = Tools::displayError('Missing video ID');
		}
	}

	public function canonicalRedirection($canonical_url = '')
	{
		if (Validate::isLoadedObject($this->category))
			parent::canonicalRedirection($this->category->getLink($this->context->language->id));
	}
	
	public function initContent()
	{
		parent::initContent();

		if (version_compare(_PS_VERSION_, '1.6.0.0') >= 0)
			$this->setTemplate(_PS_ROOT_DIR_.'/modules/videos/views/templates/front/video-category_16.tpl');
		else
			$this->setTemplate(_PS_ROOT_DIR_.'/modules/videos/views/templates/front/video-category_15.tpl');

		$categories = VideoCategory::getVideoCategories($this->context->language->id);
		foreach ($categories as &$category)
		{
			$category_obj = new VideoCategory($category['id_video_category'], $this->context->language->id);
			$category['link'] = $category_obj->link;
		}

		if (Validate::isLoadedObject($this->category))
			$videos = $this->category->getVideos($this->context->language->id);
		else
			$videos = VideoCategory::getAllVideos($this->context->language->id);

		foreach ($videos as &$item)
		{
			$item['image'] = (!file_exists(_PS_IMG_DIR_.'videos/'.$item['id_video'].'.jpg')) ? $this->context->language->iso_code.'-default.jpg' : $item['id_video'].'.jpg';
			$video = new Video($item['id_video'], $this->context->language->id);
			$item['link'] = $video->getlink($this->context->language->id);
		}

		$this->context->smarty->assign(array(
			'path' => ($this->category ? Tools::safeOutput($this->category->name) : false),
			'category' => (isset($this->category) ? $this->category : false),
			'meta_title' => (isset($this->category->meta_title) && $this->category->meta_title != '' ? $this->category->meta_title : $this->category->name),
			'meta_description' => (isset($this->category->meta_desription) && $this->category->meta_description != '' ? $this->category->meta_description : ''),
			'categories'=> $categories,
			'videos' => $videos,
			'id_category' => (isset($this->category) ? (int)$this->category->id : 0),
			'lang' => $this->context->language->id 
		));
	}

}
