<?php
/**
*  This file is part of the 'Videos' module feature developped for Prestashop  platform.
* 
*  You are not allowed to use it on several site
*  You are not allowed to sell or redistribute this module
*  This header must not be removed
* 
*  @author    Botigues.cat <info@botigues.cat>
*  @copyright 2014 Botigues.cat
*  @version  1.5.1
*/

if (!defined('_CAN_LOAD_FILES_') || !defined('_PS_VERSION_'))
	exit;

class Videos extends Module
{

	public function __construct()
	{
		$this->name = 'videos';
		$this->tab = 'front_office_features';
		$this->module_key = 'f44112b3e470f155a0aee61a45dec38d';

		parent::__construct();

		$this->version = '1.5.1';
		$this->author = 'Botigues.cat';
		$this->displayName = $this->l('Videos Manager');
		$this->description = $this->l('Manage videos');
		$this->confirmUninstall = $this->l('Are you sure you want to delete all your videos ?');

		self::$classInModule['adminvideoscontroller'] = true;
	}

	public function install()
	{
		if (!parent::install())
			return false;

		$this->createAdminTabs();
		$this->generateRewriteRules();

		if (!$this->installSQL() || !$this->installFiles() || !$this->registerHook('displayHomeTabContent') || !$this->registerHook('displayProductTabContent') || !$this->registerHook('displayProductTab'))
		{
			$this->uninstall();
			return false;
		}

		return true;
	}

	public function uninstall()
	{ 
		/**** delete AdminVideo tab ****/
		$tab_id = Tab::getIdFromClassName('AdminVideo');
		if ($tab_id)
		{
			$tab = new Tab($tab_id);
			$tab->delete();
		}

		/**** delete AdminVideos tab ****/
		$tab_id = Tab::getIdFromClassName('AdminVideos');
		if ($tab_id)
		{
			$tab = new Tab($tab_id);
			$tab->delete();
		}

		/**** delete AdminVideoCategory tab ****/
		$tab_id = Tab::getIdFromClassName('AdminVideoCategories');
		if ($tab_id)
		{
			$tab = new Tab($tab_id);
			$tab->delete();
		}

		if (!parent::uninstall() || !$this->uninstallFiles() || !Db::getInstance()->Execute('DROP TABLE '._DB_PREFIX_.'video, '._DB_PREFIX_.'video_lang, '._DB_PREFIX_.'video_shop, '._DB_PREFIX_.'video_category, '._DB_PREFIX_.'video_category_lang, '._DB_PREFIX_.'video_category_shop, '._DB_PREFIX_.'video_categories, '._DB_PREFIX_.'video_product'))
				return false;

		return true;
	}

	private function generateRewriteRules()
	{
		if (Configuration::get('PS_REWRITING_SETTINGS'))
		{
			$rules = "#start_videos - not remove this comment \n";

			$physical_uri = array();
			foreach (ShopUrl::getShopUrls() as $shop_url)
			{
				if (in_array($shop_url->physical_uri, $physical_uri))
					continue;

				$rules .= 'RewriteRule ^(.*)video$ '.$shop_url->physical_uri."index.php?controller=video [QSA,L] \n"; 
				$rules .= 'RewriteRule ^(.*)videos$ '.$shop_url->physical_uri."index.php?controller=video-category [QSA,L] \n"; 
				$rules .= 'RewriteRule ^(.*)video/([0-9]+)\-([a-zA-Z0-9-]*) '.$shop_url->physical_uri."index.php?controller=video&id_video=$2 [QSA,L] \n";
				$rules .= 'RewriteRule ^(.*)videos/([0-9]+)\-([a-zA-Z0-9-]*) '.$shop_url->physical_uri."index.php?controller=video-category&id_video_category=$2 [QSA,L] \n";

				$physical_uri[] = $shop_url->physical_uri;
			}
			$rules .= "#end_videos \n\n";

			$path = _PS_ROOT_DIR_.'/.htaccess';
			if (is_writable($path))
			{
				$existingRules = Tools::file_get_contents($path);
				if (!strpos($existingRules, 'start_videos'))
				{
					$handle = fopen($path, 'w');
					fwrite($handle, $rules.$existingRules);
					fclose($handle);
				}
			}
		}
	}

	private function installSQL()
	{
		if (!Db::getInstance()->Execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'video` (
			`id_video` int(11) NOT NULL AUTO_INCREMENT,
			`active` tinyint(1) NOT NULL,
			`code` varchar(128) NOT NULL,
			`type` enum(\'youtube\',\'vimeo\') NOT NULL DEFAULT \'youtube\',
			PRIMARY KEY (`id_video`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8'))
				return false;

		if (!Db::getInstance()->Execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'video_lang` (
			`id_video` int(10) unsigned NOT NULL AUTO_INCREMENT,
			`id_lang` int(10) unsigned NOT NULL,
			`name` varchar(128) NOT NULL,
			`description_short` varchar(255),
			`description` text,
			`link_rewrite` varchar(128) NOT NULL,
			`meta_description` varchar(255) DEFAULT NULL,
			`meta_title` varchar(128) DEFAULT NULL,
			PRIMARY KEY (`id_video`,`id_lang`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8'))
				return false;

		if (!Db::getInstance()->Execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'video_shop` (
			`id_video` INT(11) NOT NULL,
			`id_shop` INT(11) NOT NULL,
			PRIMARY KEY (`id_video`,`id_shop`),
			UNIQUE KEY `id_video_shop` (`id_video`,`id_shop`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8'))
				return false;


		if (!Db::getInstance()->Execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'video_categories` (
			`id_video` int(10) unsigned NOT NULL,
			`id_video_category` int(10) unsigned NOT NULL,
			PRIMARY KEY (`id_video`,`id_video_category`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;'))
				return false;

		if (!Db::getInstance()->Execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'video_product` (
			`id_video` int(10) unsigned NOT NULL,
			`id_product` int(10) unsigned NOT NULL,
			PRIMARY KEY (`id_video`,`id_product`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;'))
				return false;

		if (!Db::getInstance()->Execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'video_category` (
			`id_video_category` int(10) unsigned NOT NULL AUTO_INCREMENT,
			`active` tinyint(1) unsigned DEFAULT \'1\',
			PRIMARY KEY (`id_video_category`)
			) ENGINE='._MYSQL_ENGINE_.'  DEFAULT CHARSET=utf8;'))
				return false;

		if (!Db::getInstance()->Execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'video_category_lang` (
			`id_video_category` int(10) unsigned NOT NULL AUTO_INCREMENT,
			`id_lang` int(10) unsigned NOT NULL DEFAULT \'0\',
			`name` varchar(255) NOT NULL,
			`description` text,
			`link_rewrite` varchar(128) NOT NULL,
			`meta_description` varchar(255) DEFAULT NULL,
			`meta_title` varchar(128) DEFAULT NULL,
			PRIMARY KEY (`id_video_category`,`id_lang`)
			) ENGINE='._MYSQL_ENGINE_.'  DEFAULT CHARSET=utf8;'))
				return false;

		if (!Db::getInstance()->Execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'video_category_shop` (
			`id_video_category` INT(11) NOT NULL,
			`id_shop` INT(11) NOT NULL,
			PRIMARY KEY (`id_video_category`,`id_shop`),
			UNIQUE KEY `id_video_category_shop` (`id_video_category`,`id_shop`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8'))
				return false;

		return true;
	}

	private function installFiles()
	{
		//This will overwrite an existing file. Please check first if you have another files into Override
		if (version_compare(_PS_VERSION_, '1.6.0.0') >= 0)
			$dir = _PS_MODULE_DIR_.$this->name.'/install/files_16/';
		else
			$dir = _PS_MODULE_DIR_.$this->name.'/install/files_15/';
		
		$this->copy_dir($dir, _PS_ROOT_DIR_.'/');

		unlink(_PS_ROOT_DIR_.'/cache/class_index.php'); // delete class_index for regenerate it with the new classes and controllers

		return true;
	}

	private function copy_dir($source, $target)
	{
		if (!file_exists($target) && !is_dir($target))
			mkdir($target); 

		$files = array_diff(scandir($source), array('..', '.', 'index.php'));	
		foreach ($files as $file)
		{
			if (!is_dir($source.$file))
				copy($source.$file, $target.$file);
			else
				$this->copy_dir($source.$file.'/', $target.$file.'/');
		}
	}

	private function uninstallFiles($dir = null)
	{
		if (version_compare(_PS_VERSION_, '1.6.0.0') >= 0)
			$module_basedir = _PS_MODULE_DIR_.$this->name.'/install/files_16';
		else
			$module_basedir = _PS_MODULE_DIR_.$this->name.'/install/files_15';

		if ($dir == null)
		{
			$dir = $module_basedir;
			$actual_folder = '';
		}
		else
			$actual_folder = str_replace($module_basedir, '', $dir);

		/* deletes files from root video */
		$files = array_diff(scandir($dir), array('..', '.', 'index.php'));	
		foreach ($files as $file)
		{
			if (is_dir($dir.'/'.$file))
			{
				$this->uninstallFiles($dir.'/'.$file);
				if (!in_array($file, array('controllers', 'classes', 'override', 'front', 'admin','templates')))
					@rmdir(_PS_ROOT_DIR_.'/'.$actual_folder.'/'.$file);
			}
			else
				@unlink(_PS_ROOT_DIR_.'/'.$actual_folder.'/'.$file);
		}

		return true;
	}

	private function createAdminTabs()
	{
		$langs = Language::getLanguages();
		$id_lang = (int)Configuration::get('PS_LANG_DEFAULT');

		/**** create tab publications ****/
		$tab0 = new Tab();
		$tab0->class_name = 'AdminVideo';
		$tab0->module = 'videos';
		$tab0->id_parent = 0; 
		foreach ($langs as $l)
			$tab0->name[$l['id_lang']] = $this->l('Videos');
		$tab0->save();
		$blog_tab_id = $tab0->id;

		$tab1 = new Tab();
		$tab1->class_name = 'AdminVideos';
		$tab1->module = 'videos';
		$tab1->id_parent = $blog_tab_id; 
		foreach ($langs as $l)
			$tab1->name[$l['id_lang']] = $this->l('Videos');
		$tab1->save();

		/**** create tab categories ****/
		$tab2 = new Tab();
		$tab2->class_name = 'AdminVideoCategories';
		$tab2->module = 'videos';
		$tab2->id_parent = $blog_tab_id; 
		foreach ($langs as $l)
			$tab2->name[$l['id_lang']] = $this->l('Video categories');
		$tab2->save();

		/*** RIGHTS MANAGEMENT ***/
		Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'access 
		WHERE `id_tab` = '.(int)$tab0->id.' 
		OR `id_tab` = '.(int)$tab1->id.' 
		OR `id_tab` = '.(int)$tab2->id);

		Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'module_access WHERE `id_module` = '.(int)$this->id);

		$profiles = Profile::getProfiles($id_lang);
		if (count($profiles))
		{
			foreach ($profiles as $p)
			{
				Db::getInstance()->Execute('INSERT IGNORE INTO `'._DB_PREFIX_.'access`(`id_profile`,`id_tab`,`view`,`add`,`edit`,`delete`) 
					VALUES ('.$p['id_profile'].', '.(int)$tab0->id.',1,1,1,1)');

				Db::getInstance()->Execute('INSERT IGNORE INTO `'._DB_PREFIX_.'access`(`id_profile`,`id_tab`,`view`,`add`,`edit`,`delete`) 
					VALUES ('.$p['id_profile'].', '.(int)$tab1->id.',1,1,1,1)');

				Db::getInstance()->Execute('INSERT IGNORE INTO `'._DB_PREFIX_.'access`(`id_profile`,`id_tab`,`view`,`add`,`edit`,`delete`) 
					VALUES ('.$p['id_profile'].', '.(int)$tab2->id.',1,1,1,1)');

				Db::getInstance()->execute('INSERT INTO '._DB_PREFIX_.'module_access(`id_profile`, `id_module`, `configure`, `view`)
					VALUES ('.$p['id_profile'].','.(int)$this->id.',1,1)');
			}
		}	
	}

	public function hookActionHtaccessCreate($params)
	{
		$this->generateRewriteRules();
	}

	public function hookProductTab($params)
	{
		if (version_compare(_PS_VERSION_, '1.6.0.0') < 0)
		{
			$videos = Db::getInstance()->ExecuteS('
				SELECT *
				FROM `'._DB_PREFIX_.'video` t
				LEFT JOIN `'._DB_PREFIX_.'video_lang` tl ON (t.`id_video` = tl.`id_video`)
				LEFT JOIN `'._DB_PREFIX_.'video_product` vp ON (t.`id_video` = vp.`id_video`)
				WHERE tl.`id_lang` = '.(int)$this->context->language->id.' AND vp.id_product = '.(int)Tools::getValue('id_product').' AND t.`active` = 1 ORDER BY t.`id_video` DESC');

			if (!empty($videos))
				return ($this->display(__FILE__, '/views/templates/front/tab.tpl'));
		}
	}

	public function hookProductTabContent($params)
	{
		$videos = Db::getInstance()->ExecuteS('
			SELECT *
			FROM `'._DB_PREFIX_.'video` t
			LEFT JOIN `'._DB_PREFIX_.'video_lang` tl ON (t.`id_video` = tl.`id_video`)
			LEFT JOIN `'._DB_PREFIX_.'video_product` vp ON (t.`id_video` = vp.`id_video`)
			WHERE tl.`id_lang` = '.(int)$this->context->language->id.' AND vp.id_product = '.(int)Tools::getValue('id_product').' AND t.`active` = 1 ORDER BY t.`id_video` DESC');

		if (!empty($videos))
		{
			foreach ($videos as &$item)
			{
				$item['image'] = (!file_exists(_PS_IMG_DIR_.'videos/'.$item['id_video'].'.jpg')) ? $this->context->language->iso_code.'-default.jpg' : $item['id_video'].'.jpg';
				$video = new Video($item['id_video'], $this->context->language->id);
				$item['link'] = $video->getlink($this->context->language->id);
			}

			$this->context->smarty->assign(array(
				'videos' => $videos
			));

			if (version_compare(_PS_VERSION_, '1.6.0.0') >= 0)
				return ($this->display(__FILE__, '/views/templates/front/tab-content_16.tpl'));
			else
				return ($this->display(__FILE__, '/views/templates/front/tab-content_15.tpl'));
		}
	}

} 
