<?php
/**
 * This file is part of a NewQuest Project
 *
 * (c) NewQuest <contact@newquest.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * @author    NewQuest
 * @copyright NewQuest
 * @license   NewQuest
 */

require_once dirname(__FILE__).'/Api/OzonCurlClient.php';

class OzonClientApi
{
    private $api_url = '';
    private $api_token = '';
    private $api_token_secret = '';
    private $webapp_id = '';

    /** @var OzonCurlClient $request */
    public $request;

    /**
     * OzonClientApi constructor.
     */
    public function __construct()
    {
        $this->request = new OzonCurlClient();

        $this->api_url = Configuration::get('OZON_API_URL');
        $this->api_token = Configuration::get('OZON_API_TOKEN');
        $this->api_token_secret = Configuration::get('OZON_API_TOKEN_SECRET');
        $this->webapp_id = Configuration::get('OZON_WEBAPP_TOKEN');
    }

    /**
     * Check order with API.
     * @param $order_data
     * @return bool|mixed
     */
    public function checkOrder($order_data)
    {
        $data = array(
            //--------------------
            //- Auth data
            'webapp_id' => $this->webapp_id,
            'api_token' => $this->api_token,
            'api_token_secret' => $this->api_token_secret,
            //--------------------
            //- Customer data
            'ozon_uuid' => '',
            'ip' => '',
            'email' => '',
            //--------------------
            //- Order data
            'order_id' => '',
            'order_amount' => '',
            'order_currency' => '',
            //'txn_type' => '',
            'list_sku' => '',
            'billing_address' => '',
            'billing_city' => '',
            'billing_postal' => '',
            'billing_country' => '',
            'shipping_address' => '',
            'shipping_city' => '',
            'shipping_postal' => '',
            'shipping_country' => '',
        );
        foreach ($data as $field => $value) {
            if (isset($order_data[$field])) {
                $data[$field] = $order_data[$field];
            }
        }

        $result = $this->sendQuery('orders/check/', $data);

        if (!$result || !is_array($result) || !isset($result['success'])) {
            return false;
        }

        return $result;
    }

    /**
     * Send API query.
     * @param $url
     * @param $data
     * @param $type
     * @return mixed
     */
    private function sendQuery($url, $data = null, $type = 'POST')
    {
        $headers = array();

        $base_url = $this->api_url;
        return $this->request($type, trim($base_url, " /").'/'.$url, $data, $headers);
    }

    /**
     * @param $type
     * @param $url
     * @param array $data
     * @param array $headers
     * @return mixed
     */
    private function request($type, $url, array $data = null, array $headers = array())
    {
        $this->request->{$type}($url, $data, $headers);

        return $this->request->getResponse();
    }
}
