<?php
/**
 * This file is part of a NewQuest Project
 *
 * (c) NewQuest <contact@newquest.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * @author    NewQuest
 * @copyright NewQuest
 * @license   NewQuest
 */

class OzonOrderIndex
{
    public static $table = 'ozon_order_index';

    public static $index_cache = array();

    /**
     * Get index from id_order.
     * @param $id_order
     * @param bool $refresh
     * @return mixed
     */
    public static function getIndex($id_order, $refresh = true)
    {
        if (!isset(self::$index_cache[$id_order]) || $refresh) {
            $query = 'SELECT * FROM '._DB_PREFIX_.self::$table.' WHERE id_order = '.(int)$id_order;
            self::$index_cache[$id_order] = Db::getInstance()->getRow($query);
        }

        return self::$index_cache[$id_order];
    }

    /**
     * Delete index from id_order.
     * @param $id_order
     */
    public static function deleteIndex($id_order)
    {
        $query = 'DELETE FROM '._DB_PREFIX_.self::$table.' WHERE id_order = '.(int)$id_order;
        if (Db::getInstance()->execute($query)) {
            self::unsetCache($id_order);
        }
    }

    /**
     * Clean all cache or just specified id_order.
     * @param null $id_order
     */
    public static function unsetCache($id_order = null)
    {
        if ($id_order !== null) {
            unset(self::$index_cache[$id_order]);
        } else {
            self::$index_cache = array();
        }
    }

    /**
     * Save index for specified id_order.
     * @param $id_order
     * @param $success
     * @param $ozon_order_id
     * @param $score
     * @param $order_risk
     * @param $proxy_score
     * @param $ip_user_type
     * @param $high_risk_email
     * @param $proxy_anonymous
     * @param $message
     * @param array $tags
     * @return bool
     */
    public static function saveIndex(
        $id_order,
        $success,
        $ozon_order_id,
        $score,
        $order_risk,
        $proxy_score,
        $ip_user_type,
        $high_risk_email,
        $proxy_anonymous,
        $message,
        $tags = array()
    ) {
        $tags_id = array();
        if ($tags && is_array($tags)) {
            foreach ($tags as $tag_name) {
                if (!empty($tag_name)) {
                    $id_tag = OzonOrderTag::getByName($tag_name);
                    if ((int)$id_tag) {
                        $tags_id[] = (int)$id_tag;
                    }
                }
            }
        }

        $query = 'REPLACE INTO '._DB_PREFIX_.self::$table.' (`id_order`, `success`, `ozon_order_id`, `score`,
        `order_risk`, `proxy_score`, `ip_user_type`, `high_risk_email`, `proxy_anonymous`, `message`, `tags`,
        `date_add`)
		VALUES ('.(int)$id_order.', '.(int)$success.', "'.pSQL($ozon_order_id).'", '.(float)$score.
            ', "'.pSQL($order_risk).'", '.(float)$proxy_score.',"'.pSQL($ip_user_type).'", '.(int)$high_risk_email.
            ', '.(int)$proxy_anonymous.', "'.pSQL($message).'", "'.pSQL(implode(':', $tags_id)).
            '", "'.date('Y-m-d H:i:s').'")';
        if (!Db::getInstance()->Execute($query)) {
            return false;
        }

        self::unsetCache($id_order);
        return true;
    }
}
