<?php
/**
 * This file is part of a NewQuest Project
 *
 * (c) NewQuest <contact@newquest.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * @author    NewQuest
 * @copyright NewQuest
 * @license   NewQuest
 */

class OzonOrderTag extends ObjectModel
{
    public $name;

    public $date_add;

    public static $definition = array(
        'table' => 'ozon_order_tag',
        'primary' => 'id_ozon_tag',
        'fields' => array(
            'name' => array( 'type' => self::TYPE_STRING,'validate' => 'isString'),
            'date_add' => array('type' => self::TYPE_DATE,'validate' => 'isDate'),
        ),
    );

    /**
     * Get tags with specific ids.
     * @param $ids
     * @return mixed
     */
    public static function getTags($ids)
    {
        if (empty($ids)) {
            return false;
        }

        if (!is_array($ids)) {
            $ids = array((int)$ids);
        } else {
            $ids = array_map('intval', $ids);
        }

        return Db::getInstance()->executeS(
            'SELECT * FROM '._DB_PREFIX_.self::$definition['table'].
            ' WHERE id_ozon_tag IN ('.pSQL(implode(',', $ids)).')'
        );
    }

    /**
     * Get tag from name. (Create if not exist)
     * @param $name
     * @return int
     * @throws PrestaShopException
     */
    public static function getByName($name)
    {
        $id_tag = (int)Db::getInstance()->getValue(
            'SELECT id_ozon_tag FROM '._DB_PREFIX_.self::$definition['table'].' WHERE name = "'.pSQL($name).'"'
        );

        if (!$id_tag) {
            $tag = new OzonOrderTag();
            $tag->name = $name;
            $tag->add();

            $id_tag = (int)$tag->id;
        }

        return $id_tag;
    }
}
